
#include <iostream>
#include <string>

#include "param.hh"

bool Param::usePoisson = true;

unsigned int Param::stationCount = 8192;
double Param::sampleRate =        400.0;

unsigned int Param::sources = 1;

double Param::detectorDeviation = 800.0;
double Param::heightDeviation =     0.8;
double Param::timeDeviation =       0.001;

double Param::sourceOffset =   1024.0;
double Param::sourceAmplitude =   2.0;
double Param::sourceFrequency = 100.0;

double Param::radiusOfEarth =  6378136.0;	/* at the equator */
double Param::speedOfLight = 299792458.0;

void
Param::ReadParams( istream& in )
{
    for ( ;; ) {
	char buf[8192];
	in >> buf;

	if ( !in.good() ) {
	    break;
	}

	string name( buf );
	if ( name == "usePoisson" ) {
	    usePoisson = true;
	} else if ( name == "noUsePoisson" ) {
	    usePoisson = false;
	} else if ( name == "stationCount" ) {
	    in >> stationCount;
	} else if ( name == "sampleRate" ) {
	    in >> sampleRate;
	} else if ( name == "sources" ) {
	    in >> sources;
	} else if ( name == "detectorDeviation" ) {
	    in >> detectorDeviation;
	} else if ( name == "heightDeviation" ) {
	    in >> heightDeviation;
	} else if ( name == "timeDeviation" ) {
	    in >> timeDeviation;
	} else if ( name == "sourceOffset" ) {
	    in >> sourceOffset;
	} else if ( name == "sourceAmplitude" ) {
	    in >> sourceAmplitude;
	} else if ( name == "sourceFrequency" ) {
	    in >> sourceFrequency;
	} else if ( name == "radiusOfEarth" ) {
	    in >> radiusOfEarth;
	} else if ( name == "speedOfLight" ) {
	    in >> speedOfLight;
	} else {
	    cerr << "Unknown param: " << name << endl;
	}
    }
}

void
Param::WriteParams( ostream& out )
{
    if ( usePoisson ) {
	out << "# usePoisson" << endl;
    } else {
	out << "# noUsePoisson" << endl;
    }

    out << "# stationCount " << stationCount << endl;
    out << "# sampleRate " << sampleRate << endl;
    out << "# sources " << sources << endl;
    out << "# detectorDeviation " << detectorDeviation << endl;
    out << "# heightDeviation " << heightDeviation << endl;
    out << "# timeDeviation " << timeDeviation << endl;
    out << "# sourceOffset " << sourceOffset << endl;
    out << "# sourceAmplitude " << sourceAmplitude << endl;
    out << "# sourceFrequency " << sourceFrequency << endl;
    out << "# radiusOfEarth " << radiusOfEarth << endl;
    out << "# speedOfLight " << speedOfLight << endl;
}
