
#include <iostream>
#include <math.h>
#include <newran.h>

#include "param.hh"
#include "source.hh"
#include "station.hh"

int
compare( const void* a, const void* b )
{
    double aa = *(const double*)a;
    double bb = *(const double*)b;
    double dif = aa - bb;

    if ( dif > 0.000001 ) {
	return 1;
    } else if ( dif < -0.000001 ) {
	return -1;
    } else {
	return 0;
    }
}

int
main( void )
{
    Random::Set(0.46875);
    Normal norm;

    Param::ReadParams( cin );
    Param::WriteParams( cout );

    Source* srcs = 0;

    if ( Param::sources > 0 ) {
	srcs = new Source[ Param::sources ];

	Source mainSource( 0.0, 0.0, 1.0 );
	srcs[0] = mainSource;
    }

    for ( unsigned int kk=10; kk < 1000; kk += 10 ) {
	double errorAccum = 0.0;
	unsigned int samples = 0;

	Param::stationCount = kk;

	Station* stations = new Station[ Param::stationCount ];
	double* detected = new double[ Param::stationCount ];

	for ( unsigned int ii=0; ii < Param::stationCount; ++ii ) {
	    stations[ ii ].init( norm );
	}

	double scale = Param::sampleRate
	    / ( Param::sourceAmplitude * Param::sourceFrequency );

	for ( double tt = 0.0; tt < 16.0 * M_PI / Param::sourceFrequency;
	tt += 2.0 * M_PI / Param::sampleRate ) {
	    double signal = Param::sourceOffset / Param::sampleRate;
	    unsigned int dd = 0;

	    if ( Param::sources > 0 ) {
		signal += srcs[0].signal( tt );
	    }

	    for ( unsigned int ii=0; ii < Param::stationCount; ++ii ) {
		detected[dd] = stations[ii].detect( srcs, Param::sources, tt );
		if ( detected[dd] >= -0.5 ) {
		    ++dd;
		}
	    }

	    qsort( detected, dd, sizeof(double), compare );
	    double res = detected[ dd/2 ];

	    double err = ( signal - res );
	    errorAccum += err * err;
	    ++samples;

#if 0
	    cout
		<< tt
		<< " "
		<< res * scale
		<< endl;
#endif
	}

	cout << kk << " " << errorAccum / samples << endl;

	delete[] detected;
	delete[] stations;
    }

    delete[] srcs;

    return 0;
}
