
#include <iostream>
#include <newran.h>
#include <math.h>

#include "param.hh"
#include "source.hh"

Source::Source()
{
    Normal norm;

    for ( ;; ) {
	double xx = norm.Next();
	double yy = norm.Next();
	double zz = norm.Next();
	double mag2 = xx*xx + yy*yy + zz*zz;

	if ( mag2 > 0.000001 ) {
	    double mag = 1.0 / sqrt( mag2 );

	    dx = xx * mag;
	    dy = yy * mag;
	    dz = zz * mag;

	    break;
	}
    }

    Uniform uu;
    periodOffset = 2.0 * M_PI * uu.Next();
    amplitude = 4.0 * Param::sourceAmplitude * uu.Next()
    	- 2.0 * Param::sourceAmplitude;
    frequency = 1.5 * Param::sourceFrequency * uu.Next()
    	+ 0.1 * Param::sourceFrequency;
}

Source::Source( double dxx, double dyy, double dzz )
{
    dx = dxx;
    dy = dyy;
    dz = dzz;

    periodOffset = 0.0;
    amplitude = Param::sourceAmplitude;
    frequency = Param::sourceFrequency;
}

Source::Source( const Source& a )
{
    dx = a.dx;
    dy = a.dy;
    dz = a.dz;

    periodOffset = a.periodOffset;
    amplitude = a.amplitude;
    frequency = a.frequency;
}

double
Source::signal( double t )
{
	/*
	** Integral of  A + B sin( t )
	**  is:  At - B cos( t )
	*/
    double t1 = t - 0.5 / Param::sampleRate;
    double t2 = t + 0.5 / Param::sampleRate;

    double a1 = frequency * t1 + periodOffset;
    double a2 = frequency * t2 + periodOffset;
    
    double ideal =
	- amplitude * ( cos( a2 ) - cos( a1 ) )
	    / frequency;

    return ideal;
}
