
#include <iostream>
#include <math.h>
#include <newran.h>

#include "param.hh"
#include "source.hh"
#include "station.hh"

void
Station::init( Normal& nn )
{
    norm = &nn;

    double s3 = sqrt(3.0);

    ex = Param::heightDeviation * norm->Next() / s3;
    ey = Param::heightDeviation * norm->Next() / s3;
    ez = Param::heightDeviation * norm->Next() / s3;

    for ( ;; ) {
	double xx = norm->Next();
	double yy = norm->Next();
	double zz = norm->Next();
	double mag2 = xx*xx + yy*yy + zz*zz;

	if ( mag2 > 0.000001 ) {
	    double mag = Param::radiusOfEarth / sqrt( mag2 );

	    x = xx * mag;
	    y = yy * mag;
	    z = zz * mag;

		/*
		** hack to keep all of the detectors in the
		** northern hemisphere
		*/
	    if ( x < 0.0 ) {
		z = -z;
	    }

	    break;
	}
    }

    timeOffset = z / Param::speedOfLight;
}

double
Station::detect( Source* srcs, unsigned int nSrcs, double t )
{
    double ideal = Param::sourceOffset / Param::sampleRate;

    for ( unsigned int ii=0; ii < nSrcs; ++ii ) {
	double srcDT = ( srcs[ii].dx * x + srcs[ii].dy * y + srcs[ii].dz * z )
	    / Param::speedOfLight;

	double eh = ( srcs[ii].dx * ex + srcs[ii].dy * ey + srcs[ii].dz * ez )
	    / Param::speedOfLight;

	double et = Param::timeDeviation * norm->Next();

	if ( srcDT > 0.0 ) {
	    ideal += srcs[ii].signal( t + timeOffset - srcDT + eh + et );
	} else if ( ii == 0 ) {
		/* bail if this detector cannot possibly see the primary */
	    return -1.0;
	}
    }

    double detectorNoise =
	norm->Next() * Param::detectorDeviation / Param::sampleRate;

    if ( Param::usePoisson ) {
	Poisson pp( ideal );
	return pp.Next() + detectorNoise;
    } else {
	return ideal + detectorNoise;
    }
}
